package com.example.gallery.repository

import com.example.gallery.models.Album
import com.example.gallery.remote.RetrofitClient
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

class GalleryRepository {
    private val apiService = RetrofitClient.apiService
    private val photoCache = PhotoCache()

    suspend fun getAlbums(): Result<List<Album>> = withContext(Dispatchers.IO) {
        try {
            val response = apiService.getAlbums()
            photoCache.saveAlbums(response)
            Result.success(response)
        } catch (e: Exception) {
            val cachedAlbums = photoCache.getAlbums()
            if (cachedAlbums != null) {
                Result.success(cachedAlbums)
            } else {
                Result.failure(e)
            }
        }
    }

    suspend fun getAlbumDetails(albumId: Int): Result<Album> = withContext(Dispatchers.IO) {
        try {
            val response = apiService.getAlbumDetails(albumId)
            photoCache.saveAlbumDetails(response)
            Result.success(response)
        } catch (e: Exception) {
            val cachedAlbum = photoCache.getAlbumDetails(albumId)
            if (cachedAlbum != null) {
                Result.success(cachedAlbum)
            } else {
                Result.failure(e)
            }
        }
    }
}
