package com.example.gallery.repository

import android.content.Context
import com.example.gallery.App
import com.example.gallery.models.Album
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken

class PhotoCache {
    companion object {
        private const val ALBUMS_CACHE_KEY = "albums_cache"
        private const val ALBUM_DETAILS_CACHE_KEY = "album_details_cache_"
    }

    private val sharedPreferences = App.instance.getSharedPreferences("photo_cache", Context.MODE_PRIVATE)

    fun saveAlbums(albums: List<Album>) {
        val json = Gson().toJson(albums)
        sharedPreferences.edit().putString(ALBUMS_CACHE_KEY, json).apply()
    }

    fun getAlbums(): List<Album>? {
        val json = sharedPreferences.getString(ALBUMS_CACHE_KEY, null) ?: return null
        val type = object : TypeToken<List<Album>>() {}.type
        return try {
            Gson().fromJson(json, type)
        } catch (e: Exception) {
            null
        }
    }

    fun saveAlbumDetails(album: Album) {
        val json = Gson().toJson(album)
        sharedPreferences.edit().putString(ALBUM_DETAILS_CACHE_KEY + album.id, json).apply()
    }

    fun getAlbumDetails(albumId: Int): Album? {
        val json = sharedPreferences.getString(ALBUM_DETAILS_CACHE_KEY + albumId, null) ?: return null
        return try {
            Gson().fromJson(json, Album::class.java)
        } catch (e: Exception) {
            null
        }
    }
}