package com.example.gallery.ui.albums

import android.content.Context
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.gallery.App
import com.example.gallery.models.Album
import com.example.gallery.repository.GalleryRepository
import kotlinx.coroutines.launch

class AlbumsViewModel : ViewModel() {
    private val repository = GalleryRepository()

    private val _albums = MutableLiveData<List<Album>>()
    val albums: LiveData<List<Album>> = _albums

    private val _isLoading = MutableLiveData<Boolean>()
    val isLoading: LiveData<Boolean> = _isLoading

    private val _error = MutableLiveData<String?>()
    val error: LiveData<String?> = _error

    private val _isFromCache = MutableLiveData<Boolean>()
    val isFromCache: LiveData<Boolean> = _isFromCache

    init {
        loadAlbums()
    }

    fun loadAlbums() {
        viewModelScope.launch {
            _isLoading.value = true
            _error.value = null
            _isFromCache.value = false

            repository.getAlbums().fold(
                onSuccess = { albumsList ->
                    _albums.value = albumsList
                    _isLoading.value = false

                    if (albumsList.isNotEmpty() && !isInternetAvailable()) {
                        _isFromCache.value = true
                    }
                },
                onFailure = { e ->
                    _error.value = e.message ?: "Неизвестная ошибка"
                    _isLoading.value = false
                }
            )
        }
    }

    fun refreshAlbums() {
        loadAlbums()
    }

    private fun isInternetAvailable(): Boolean {
        val connectivityManager = App.instance.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val network = connectivityManager.activeNetwork ?: return false
        val networkCapabilities = connectivityManager.getNetworkCapabilities(network) ?: return false
        return networkCapabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
    }
}